import unittest

import tsaugmentation as tsag

from gpforecaster.model.gpf import GPF
from gpforecaster.visualization import plot_predictions_vs_original


class TestModel(unittest.TestCase):
    def setUp(self):
        self.data = tsag.preprocessing.PreprocessDatasets(
            "prison", sample_perc=0.9, freq='Q'
        ).apply_preprocess()
        self.n = self.data["predict"]["n"]
        self.s = self.data["train"]["s"]
        self.res_type = 'fitpred'
        self.res_measure = 'mean'
        self.input_dir = './results/gpf/'
        self.dataset_name = "prison"
        self.gpf = GPF("prison", self.data, input_dir=self.input_dir, gp_type="exact90")

    def test_calculate_metrics_dict(self):
        model, like = self.gpf.train(epochs=10)
        preds, preds_scaled = self.gpf.predict(model, like)
        plot_predictions_vs_original(
            dataset=self.dataset_name,
            prediction_mean=preds[0],
            prediction_std=preds[1],
            origin_data=self.gpf.original_data,
            x_original=self.gpf.train_x.numpy(),
            x_test=self.gpf.test_x.numpy(),
            inducing_points=self.gpf.inducing_points,
            n_series_to_plot=8,
            gp_type=self.gpf.gp_type,
        )
        res = self.gpf.metrics(preds[0], preds[1])
        self.assertLess(res['mase']['bottom'], 20)



import pickle
import unittest

import tsaugmentation as tsag

from gpforecaster.model.gpf import GPF
from gpforecaster.visualization import plot_predictions_vs_original


class TestModel(unittest.TestCase):
    def setUp(self):
        self.dataset_name = "prison"
        self.data = tsag.preprocessing.PreprocessDatasets(
            self.dataset_name, freq="Q"
        ).apply_preprocess()
        self.data_minmax = tsag.preprocessing.PreprocessDatasets(
            self.dataset_name, freq="Q"
        ).apply_preprocess()
        self.data_sparse = tsag.preprocessing.PreprocessDatasets(
            self.dataset_name, freq="Q"
        ).apply_preprocess()
        self.n = self.data["predict"]["n"]
        self.s = self.data["train"]["s"]
        self.res_type = "fitpred"
        self.res_measure = "mean"
        self.input_dir = "./results/gpf/"
        self.gpf = GPF(
            self.dataset_name,
            self.data,
            input_dir=self.input_dir,
            scaler_type="standard",
            scale_x_values=True,
        )
        self.gpf_sparse = GPF(
            self.dataset_name,
            self.data_sparse,
            input_dir=self.input_dir,
            scaler_type="standard",
            scale_x_values=True,
            gp_type="sparse"
        )
        self.gpf_minmax = GPF(
            self.dataset_name,
            self.data_minmax,
            input_dir=self.input_dir,
            scaler_type="minmax",
            scale_x_values=True,
            gp_type="exact"
        )

    def test_train_scaled_no_validation_or_cross(self):
        model, like = self.gpf.train(
            lr=0.01,
            weight_decay=0.0001,
            epochs=11,
            patience=5,
            random_init=True,
            cross_validation=False,
            no_validation=True,
        )
        self.gpf.plot_losses()
        preds, preds_scaled = self.gpf.predict(model, like)
        plot_predictions_vs_original(
            dataset=self.dataset_name,
            prediction_mean=preds[0],
            prediction_std=preds[1],
            origin_data=self.gpf.original_data,
            x_original=self.gpf.train_x,
            x_test=self.gpf.test_x,
            inducing_points=self.gpf.inducing_points,
            n_series_to_plot=8,
            gp_type=self.gpf.gp_type,
        )
        self.assertIsNotNone(model)

    def test_train_scaled_no_validation_or_cross_minmax(self):
        model, like = self.gpf_minmax.train(
            lr=0.01,
            weight_decay=0.0001,
            epochs=11,
            patience=5,
            random_init=True,
            cross_validation=False,
            no_validation=True,
        )
        self.gpf_minmax.plot_losses()
        preds, preds_scaled = self.gpf_minmax.predict(model, like)
        plot_predictions_vs_original(
            dataset=self.dataset_name,
            prediction_mean=preds[0],
            prediction_std=preds[1],
            origin_data=self.gpf_minmax.original_data,
            x_original=self.gpf_minmax.train_x,
            x_test=self.gpf_minmax.test_x,
            inducing_points=self.gpf_minmax.inducing_points,
            n_series_to_plot=8,
            gp_type=self.gpf_minmax.gp_type,
        )
        self.assertIsNotNone(model)

    def test_train_scaled_no_validation_or_cross_sparse(self):
        model, like = self.gpf_sparse.train(
            lr=0.01,
            weight_decay=0.0001,
            epochs=11,
            patience=5,
            random_init=True,
            cross_validation=False,
            no_validation=True,
        )
        self.gpf_sparse.plot_losses()
        preds, preds_scaled = self.gpf_sparse.predict(model, like)
        plot_predictions_vs_original(
            dataset=self.dataset_name,
            prediction_mean=preds[0],
            prediction_std=preds[1],
            origin_data=self.gpf_sparse.original_data,
            x_original=self.gpf_sparse.train_x,
            x_test=self.gpf_sparse.test_x,
            inducing_points=self.gpf_sparse.inducing_points,
            n_series_to_plot=8,
            gp_type=self.gpf_sparse.gp_type,
        )
        self.assertIsNotNone(model)

    def test_predict_shape(self):
        model, like = self.gpf.train(epochs=10)
        preds, preds_scaled = self.gpf.predict(model, like)
        self.assertTrue(preds[0].shape == (self.n, self.s))

    def test_results_interval(self):
        model, like = self.gpf.train(epochs=100)
        preds, preds_scaled = self.gpf.predict(model, like)
        res = self.gpf.metrics(preds[0], preds[1])
        self.assertLess(res["mase"]["bottom"], 40)

    def test_wall_time(self):
        model, like = self.gpf.train(epochs=10)
        preds, preds_scaled = self.gpf.predict(model, like)
        res = self.gpf.metrics(preds[0], preds[1])
        self.assertLess(res["wall_time"]["wall_time_total"], 100)

    def test_output_results(self):
        model, like = self.gpf.train(epochs=10)
        preds, preds_scaled = self.gpf.predict(model, like)
        res = self.gpf.metrics(preds[0], preds[1])
        self.gpf.store_results(
            res, res_type=self.res_type, res_measure=self.res_measure
        )
        with open(
            f"{self.gpf.input_dir}results_{self.res_type}_{self.res_measure}_gp_{self.gpf.gp_type}_cov_{self.gpf.dataset}_{self.gpf.model_version}.pickle",
            "rb",
        ) as handle:
            output_res = pickle.load(file=handle)
        self.assertIsNotNone(output_res["mase"]["bottom"])

    def test_plot_loss_xvalidation(self):
        model, like = self.gpf.train(epochs=10)
        self.gpf.plot_losses()

    def test_plot_loss(self):
        model, like = self.gpf.train(epochs=10, cross_validation=False)
        self.gpf.plot_losses()

    def test_no_validation(self):
        model, like = self.gpf.train(
            epochs=10, cross_validation=False, no_validation=True
        )
        self.gpf.plot_losses()



import unittest
import tsaugmentation as tsag
from gpforecaster.model.gpf import GPF
import shutil


class TestModel(unittest.TestCase):

    def setUp(self):
        self.preproc = tsag.preprocessing.PreprocessDatasets('tourism', test_size=228*10, freq='M')
        self.data = self.preproc._tourism()
        self.n = self.data['predict']['n']
        self.s = self.data['train']['s']
        self.gpf = GPF(dataset='tourism', groups=self.data)

    def test_results_mean_and_prediction_interval(self):
        model, like = self.gpf.train(epochs=10)
        preds, preds_scaled = self.gpf.predict(model, like)
        res = self.gpf.metrics(preds[0], preds[1])

        # Test shape of results
        self.assertTrue(res['mase']['bottom_ind'].shape == (self.s, ))
        self.assertTrue(res['CRPS']['bottom_ind'].shape == (self.s, ))
        self.assertTrue(res['rmse']['bottom_ind'].shape == (self.s, ))

    def test_results_mean_and_prediction_interval_without_storing_results(self):
        self.gpf = GPF(dataset='tourism', groups=self.data)
        model, like = self.gpf.train(epochs=10)
        preds, preds_scaled = self.gpf.predict(model, like)
        res = self.gpf.metrics(preds[0], preds[1])

        # Test shape of results
        self.assertTrue(res['mase']['bottom_ind'].shape == (self.s, ))
        self.assertTrue(res['CRPS']['bottom_ind'].shape == (self.s, ))
        self.assertTrue(res['rmse']['bottom_ind'].shape == (self.s, ))

        # Test shape of predictions
        # Test number of objects predicted and stored
        self.assertTrue(len(res) == 4)




from typing import Tuple, Dict, List, Any, Optional
import gc
import numpy as np

from skopt import Optimizer
from skopt.space import Real, Categorical, Integer
from skopt.utils import use_named_args

from gpforecaster.model.gpf import GPF
from gpforecaster.utils.logger import Logger
from gpforecaster import __version__


def single_trial(
    epochs: int,
    dataset_name: str,
    hierarchical_data: Dict[str, List],
    hyperparameters: Dict[str, Any],
    gp_type: str,
    device: str,
    logger_tuning: Logger,
    trial_num: int,
) -> Tuple[float, float, Dict[str, Any]]:
    """
    Train a single GPF model with the given hyperparameters.

    Args:
        dataset_name (str): The name of the dataset.
        hierarchical_data (Dict[str, List]): The hierarchical data for the model.
        hyperparameters (Dict[str, Any]): The hyperparameters to use for the model.
        gp_type (str): The type of Gaussian Process model to use (e.g., "exact").
        device (str): The device to use for computation (e.g., "cpu" or "cuda").
        logger_tuning (Logger): The logger for hyperparameter tuning.
        trial_num (int): The trial number (for display purposes).

    Yields:
        Tuple[float, Dict[str, Any]]: A tuple containing the validation loss and the hyperparameters used.
    """
    print(f"Running trial number {trial_num}")
    logger_tuning.info(f"Running trial number {trial_num}")

    gpf = GPF(
        dataset=dataset_name,
        groups=hierarchical_data,
        gp_type=gp_type,
        device=device,
        scaler_type=hyperparameters["scaler_type"],
        scale_x_values=hyperparameters["scale_x_values"],
    )

    model, like = None, None

    penalty = 1e5

    try:
        model, like = gpf.train(
            epochs=epochs,
            lr=hyperparameters["learning_rates"],
            weight_decay=hyperparameters["weight_decays"],
            scheduler_type=hyperparameters["scheduler_types"],
            gamma_rate=hyperparameters["gamma_rates"],
            patience=hyperparameters["patiences"],
            rbf_kernel_lengthscale=hyperparameters["rbf_kernel_lengthscale"],
            scale_rbf_kernel_outputscale=hyperparameters[
                "scale_rbf_kernel_outputscale"
            ],
            periodic_kernel_lengthscale=hyperparameters["periodic_kernel_lengthscale"],
            scale_periodic_kernel_outputscale=hyperparameters[
                "scale_periodic_kernel_outputscale"
            ],
            m=hyperparameters["m"],
            k=hyperparameters["k"],
            b=hyperparameters["b"],
            random_init=hyperparameters["random_init"],
        )
        val_loss = np.mean(gpf.avg_val_loss)
        test_loss = np.mean(gpf.avg_test_loss)
        if np.isnan(val_loss):
            val_loss = penalty
            test_loss = penalty
    except:
        print(f"Error occurred during training with the current set of hyperparameters")
        val_loss = penalty
        test_loss = penalty

    log_and_print_best_hyperparameters(
        gp_type, dataset_name, hyperparameters, val_loss, test_loss, logger_tuning
    )

    if model is not None:
        del model
    if like is not None:
        del like
    del gpf
    gc.collect()

    return (val_loss, test_loss, hyperparameters)


def optimize_hyperparameters_bayesian(
    dataset_name: str,
    hierarchical_data: Dict[str, List],
    num_trials: int,
    gp_type: str = "exact",
    device: str = "cpu",
    epochs: int = 500,
    hyperparameters_list: Optional[List[str]] = None,
    fixed_hyperparameters: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    """
    Optimize hyperparameters using Bayesian search.

    Args:
        dataset_name (str): The name of the dataset.
        hierarchical_data (Dict[str, List]): The hierarchical data for the model.
        num_trials (int): The number of trials to perform.
        gp_type (str): The type of Gaussian Process model to use (e.g., "exact").

    Returns:
        Dict[str, Any]: The best set of hyperparameters found during optimization.
    """
    logger_tuning = Logger(
        "hyperparameter_tuning", dataset=f"{dataset_name}_hypertuning", to_file=True
    )

    if hyperparameters_list is None:
        hyperparameters_list = [
            "learning_rates", "weight_decays", "scheduler_types",
            "gamma_rates", "patiences", "rbf_kernel_lengthscale",
            "scale_rbf_kernel_outputscale", "periodic_kernel_lengthscale",
            "scale_periodic_kernel_outputscale", "m", "k", "b",
            "scaler_type", "scale_x_values", "random_init"
        ]

    # Define the search space for hyperparameters
    all_hyperparameters = {
        "learning_rates": Real(1e-3, 0.1, name="learning_rates"),
        "weight_decays": Real(1e-6, 1e-3, name="weight_decays"),
        "scheduler_types": Categorical(["step", "exponential", "cosine", "none"], name="scheduler_types"),
        "gamma_rates": Real(0.1, 0.95, name="gamma_rates"),
        "patiences": Integer(4, 30, name="patiences"),
        "rbf_kernel_lengthscale": Real(0.01, 2, name="rbf_kernel_lengthscale"),
        "scale_rbf_kernel_outputscale": Real(0.01, 2, name="scale_rbf_kernel_outputscale"),
        "periodic_kernel_lengthscale": Real(0.01, 2, name="periodic_kernel_lengthscale"),
        "scale_periodic_kernel_outputscale": Real(0.01, 2, name="scale_periodic_kernel_outputscale"),
        "m": Real(0.0001, 0.3, name="m"),
        "k": Real(0.0001, 0.3, name="k"),
        "b": Real(0.0001, 0.3, name="b"),
        "scaler_type": Categorical(["minmax", "standard"], name="scaler_type"),
        "scale_x_values": Categorical(["True", "False"], name="scale_x_values"),
        "random_init": Categorical(["True", "False"], name="random_init"),
    }

    if fixed_hyperparameters is not None:
        for key in fixed_hyperparameters.keys():
            all_hyperparameters.pop(key, None)

    search_space = [hp for name, hp in all_hyperparameters.items() if hp.name in hyperparameters_list]

    optimizer = Optimizer(search_space)
    trial_num = 0
    test_loss = []

    current_best_val_loss = float("inf")
    current_best_hyperparameters = None
    current_best_test_loss = None

    @use_named_args(search_space)
    def wrapped_single_trial(**hyperparameters):
        nonlocal trial_num, current_best_val_loss, current_best_hyperparameters, current_best_test_loss
        if fixed_hyperparameters is not None:
            hyperparameters = {**fixed_hyperparameters, **hyperparameters}
        val_loss, temp_test_loss, current_hyperparameters = single_trial(
            epochs,
            dataset_name,
            hierarchical_data,
            hyperparameters,
            gp_type,
            device,
            logger_tuning,
            trial_num=trial_num,
        )
        test_loss.append(temp_test_loss)
        trial_num += 1

        if val_loss < current_best_val_loss:
            current_best_val_loss = val_loss
            current_best_hyperparameters = current_hyperparameters
            current_best_test_loss = temp_test_loss

        log_and_print_best_hyperparameters(
            gp_type,
            dataset_name,
            current_best_hyperparameters,
            current_best_val_loss,
            current_best_test_loss,
            logger_tuning,
            best="CURRENT BEST",
        )
        return val_loss

    results = optimizer.run(wrapped_single_trial, num_trials)
    best_hyperparameters = {k.name: v for k, v in zip(search_space, results.x)}

    best_trial_idx = np.argmin(results.func_vals)
    best_test_loss = test_loss[best_trial_idx]

    if fixed_hyperparameters:
        best_hyperparameters = {**fixed_hyperparameters, **best_hyperparameters}

    log_and_print_best_hyperparameters(
        gp_type,
        dataset_name,
        best_hyperparameters,
        results.fun,
        best_test_loss,
        logger_tuning,
        best="BEST",
    )
    return best_hyperparameters


def log_and_print_best_hyperparameters(
    gp_type: str,
    dataset_name: str,
    best_hyperparameters: Dict[str, Any],
    best_val_loss: float,
    best_test_loss: float,
    logger_tuning: Logger,
    best="",
):
    """
    Log and print the best hyperparameters and validation loss.

    Args:
        gp_type (str): The type of Gaussian Process model used (e.g., "exact").
        dataset_name (str): The name of the dataset.
        best_hyperparameters (Dict[str, Any]): The best set of hyperparameters found.
        best_val_loss (float): The best validation loss.
    """

    logger_tuning.info(
        f"\n{best} -> "
        f"Algorithm: gpf_{gp_type}, "
        f"Version: {__version__}, "
        f"Dataset: {dataset_name}, "
        f"{best} hyperparameters: {best_hyperparameters}, "
        f"Validation loss: {best_val_loss}\n"
        f"Test loss: {best_test_loss}\n"
    )

    print(
        f"\n{best} -> "
        f"Algorithm: gpf_{gp_type}, \n"
        f"Version: {__version__}, \n"
        f"Dataset: {dataset_name}, \n"
        f"{best} hyperparameters: {best_hyperparameters}, \n"
        f"Validation loss: {best_val_loss}\n"
        f"Test loss: {best_test_loss}\n"
    )



import logging
import os
import time
from gpforecaster import __version__


class Logger:
    def __init__(self, name, dataset, to_file=None, log_level=logging.INFO, log_dir="."):
        timestamp = time.strftime("%Y%m%d-%H%M%S", time.gmtime())

        self.logger = logging.getLogger(name)
        self.logger.setLevel(log_level)

        formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

        handler = logging.StreamHandler()  # Outputs the log to the console
        if to_file:
            log_dir_path = os.path.join(log_dir, "logs")
            if not os.path.exists(log_dir_path):
                os.makedirs(log_dir_path)

            log_file = os.path.join(log_dir_path, f"gpf_{__version__}_{dataset}_log_{timestamp}.txt")
            handler = logging.FileHandler(log_file)  # Outputs the log to a file
        handler.setFormatter(formatter)

        self.logger.addHandler(handler)

    def info(self, message):
        self.logger.info(message)

    def warning(self, message):
        self.logger.warning(message)

    def error(self, message):
        self.logger.error(message)



import unittest
import numpy as np
import tsaugmentation as tsag
from gpforecaster.model.gpf import GPF
from gpforecaster.visualization import plot_predictions_vs_original


class TestModel(unittest.TestCase):
    def setUp(self):
        self.dataset_name = "prison"
        self.data = tsag.preprocessing.PreprocessDatasets(
            self.dataset_name, freq='Q'
        ).apply_preprocess()
        self.data_w_missing = tsag.preprocessing.PreprocessDatasets(
            self.dataset_name, sample_perc=0.9, freq='Q'
        ).apply_preprocess()
        self.n = self.data["predict"]["n"]
        self.s = self.data["train"]["s"]
        self.data_w_missing["predict"] = self.data["predict"]
        self.gpf = GPF(self.dataset_name, self.data_w_missing, gp_type="exact50")

    def test_calculate_metrics_dict(self):
        model, like = self.gpf.train(epochs=10)
        preds, preds_scaled = self.gpf.predict(model, like)
        plot_predictions_vs_original(
            dataset=self.dataset_name,
            prediction_mean=preds[0],
            prediction_std=preds[1],
            origin_data=self.gpf.original_data,
            x_original=np.array(self.data["train"]["x_values"])[:, np.newaxis],
            x_test=self.gpf.test_x.numpy()[:, np.newaxis],
            inducing_points=self.gpf.inducing_points,
            n_series_to_plot=8,
            gp_type=self.gpf.gp_type,
        )
        res = self.gpf.metrics(preds[0], preds[1])
        self.assertLess(res["mase"]["bottom"], 25)



import unittest
import pickle

import tsaugmentation as tsag

from gpforecaster.model.gpf import GPF
from gpforecaster import __version__


class TestModel(unittest.TestCase):
    def setUp(self):
        self.dataset_name = "prison"
        self.gp_type = "exact"
        self.version = __version__
        self.data = tsag.preprocessing.PreprocessDatasets(
            self.dataset_name, freq='Q'
        ).apply_preprocess()
        self.n = self.data["predict"]["n"]
        self.s = self.data["train"]["s"]
        self.h = self.data["h"]
        self.gpf = GPF(
            self.dataset_name,
            self.data,
            log_dir="..",
            gp_type=self.gp_type,
            inducing_points_perc=0.75,
        )
        self.gpf.input_dir = f"./results/gpf/"

    def test_store_fit_pred_mean_results_gp(self):
        model, like = self.gpf.train(
            epochs=10,
            patience=4,
            lr=1e-2,
        )
        preds, preds_scaled = self.gpf.predict(model, like)
        res_type = "fitpred"
        res_measure = "mean"
        self.gpf.store_results(preds[0], res_type=res_type, res_measure=res_measure)
        with open(
            f"{self.gpf.input_dir}results_{res_type}_{res_measure}_gp_{self.gp_type}_cov_{self.dataset_name}_{self.version}.pickle",
            "rb",
        ) as handle:
            res = pickle.load(handle)
        self.assertTrue(res.shape == (self.n, self.s))

    def test_store_pred_std_results_gp(self):
        model, like = self.gpf.train(
            epochs=10,
            patience=4,
            lr=1e-2,
        )
        preds, preds_scaled = self.gpf.predict(model, like)
        res_type = "pred"
        res_measure = "std"
        self.gpf.store_results(
            preds[1][self.n - self.h :, :], res_type=res_type, res_measure=res_measure
        )
        with open(
            f"{self.gpf.input_dir}results_{res_type}_{res_measure}_gp_{self.gp_type}_cov_{self.dataset_name}_{self.version}.pickle",
            "rb",
        ) as handle:
            res = pickle.load(handle)
        self.assertTrue(res.shape == (self.h, self.s))

    def test_store_metrics_gp(self):
        model, like = self.gpf.train(
            epochs=10,
            patience=4,
            lr=1e-2,
        )
        self.gpf.input_dir = f"./results/gpf/"
        preds, preds_scaled = self.gpf.predict(model, like)
        res = self.gpf.metrics(preds[0], preds[1])
        self.gpf.store_metrics(res)
        with open(
            f"{self.gpf.input_dir}metrics_gp_{self.gp_type}_cov_{self.dataset_name}_{self.version}.pickle",
            "rb",
        ) as handle:
            res = pickle.load(handle)
        keys = list(res.keys())
        self.assertTrue(keys == ['mase', 'rmse', 'CRPS', 'wall_time'])



import time

import matplotlib.pyplot as plt
import numpy as np

from gpforecaster import __version__


def plot_predictions_vs_original(
    dataset: str,
    prediction_mean: np.ndarray,
    prediction_std: np.ndarray,
    origin_data: np.ndarray,
    x_original: np.ndarray,
    x_test: np.ndarray,
    inducing_points: np.ndarray,
    gp_type: str,
    n_series_to_plot: int = 8,
) -> None:
    """
    Plot predictions and original series and store as pdf

    Parameters:
        dataset: dataset to plot
        prediction_mean: prediction mean (n_samples, n_series)
        prediction_std: prediction std (n_samples, n_series)
        origin_data: original dataset with shape (n_samples, n_series)
        inducing_points: inducing points to plot if sparse GP was used
        gp_type: type of GP used (exact, sparse, etc)
        n_series_to_plot: number of series to plot
    """
    has_inducing_points = True
    if has_inducing_points is None:
        has_inducing_points = False
    mean = prediction_mean
    upper = prediction_mean + 1.96 * prediction_std
    lower = prediction_mean - 1.96 * prediction_std
    n_train = len(x_original)
    n_test = len(x_test)

    x_total = np.squeeze(np.concatenate((x_original, x_test)))

    # n_series needs to be even
    if not n_series_to_plot % 2 == 0:
        n_series_to_plot -= 1

    _, ax = plt.subplots(n_series_to_plot // 2, 2, figsize=(20, 10))
    ax = ax.ravel()

    for i, ax_ in enumerate(ax):
        ax_.scatter(x_original, origin_data[:n_train, i], label="Training data")
        ax_.scatter(x_test, origin_data[-n_test:, i], label="Testing data")
        ax_.plot(x_total, mean[:, i], label="GP mean")
        ax_.fill_between(
            x_total, lower[:, i], upper[:, i], alpha=0.5, label="GP uncertainty"
        )
        if has_inducing_points:
            ax[i].scatter(
                inducing_points,
                np.zeros((inducing_points.shape[0],)),
                label="Inducing points",
            )
    plt.suptitle(f"Predictions for {dataset} using {gp_type} GPs")
    plt.legend()
    timestamp = time.strftime("%Y%m%d-%H%M%S", time.gmtime())
    plt.savefig(
        f"./plots/gpf_{__version__}_preds_{dataset}_{timestamp}.pdf",
        format="pdf",
        bbox_inches="tight",
    )
    plt.show()



__version__ = "0.3.116"

from gpforecaster import model
from gpforecaster import results
from gpforecaster import visualization

# Only print in interactive mode
import __main__ as main
if not hasattr(main, '__file__'):
    print("""Importing the gpforecaster module. L. Roque. 
    Algorithm to forecast Hierarchical Time Series providing point forecast and uncertainty intervals.\n""")
